function contentScript() {
  // by passing an object you can define default values e.g.: []
  chrome.storage.local.get({ preBlurElements: [] }, function (result) {
    // the input argument is ALWAYS an object containing the queried keys
    // so we select the key we need
    var preBlurElements = result.preBlurElements;
    console.log(
      `preBlurElements: ${preBlurElements} length: ${preBlurElements.length}`
    );

    try {
      for (var i = 0; i < preBlurElements.length; i++) {
        if (document.URL === preBlurElements[i]["url"]) {
          var xpath = preBlurElements[i]["xpath"];
          console.log(`xpath: ${xpath}`);

          //--- update blur value
          if (preBlurElements[i]["blur"]) {
            document.documentElement.style.setProperty(
              "--blur-value",
              `${preBlurElements[i]["blur"]}px`
            );
          } else {
            document.documentElement.style.setProperty("--blur-value", `10px`);
          }

          var matchingElement = document.evaluate(
            xpath,
            document,
            null,
            XPathResult.FIRST_ORDERED_NODE_TYPE,
            null
          ).singleNodeValue;
          //matchingElement.style.filter = "blur(15px)";
          matchingElement.classList.add("__blurweb-blur");
        }
      }
      document.body.style.visibility = "visible";
    } catch (error) {
      document.body.style.visibility = "visible";
      console.log(`error: ${error}`);
    }
  });

  // by passing an object you can define default values e.g.: []
  chrome.storage.local.get({ preBlurAreasInfo: [] }, function (result) {
    // the input argument is ALWAYS an object containing the queried keys
    // so we select the key we need
    var preBlurAreasInfo = result.preBlurAreasInfo;

    try {
      for (var i = 0; i < preBlurAreasInfo.length; i++) {
        if (document.URL === preBlurAreasInfo[i]["url"]) {
          if (preBlurAreasInfo[i]["blur"]) {
            document.documentElement.style.setProperty(
              "--blur-value",
              `${preBlurAreasInfo[i]["blur"]}px`
            );
          } else {
            document.documentElement.style.setProperty("--blur-value", `10px`);
          }

          var id = preBlurAreasInfo[i]["id"];
          var left = preBlurAreasInfo[i]["left"];
          var top = preBlurAreasInfo[i]["top"];
          var height = preBlurAreasInfo[i]["height"];
          var width = preBlurAreasInfo[i]["width"];

          const deleteBlurwebBlurAreaBtn = document.createElement("div");
          deleteBlurwebBlurAreaBtn.id = "fig_delete_id_2000";
          deleteBlurwebBlurAreaBtn.classList.add("__blurweb-area-blur-delete");
          deleteBlurwebBlurAreaBtn.style.display = "none";
          deleteBlurwebBlurAreaBtn.innerText = "delete";

          const blurwebBlurAreaEl = document.createElement("div");
          blurwebBlurAreaEl.id = id;
          blurwebBlurAreaEl.style.backdropFilter = `blur(var(--blur-value))`;
          blurwebBlurAreaEl.className = "drawing __blurweb-area-blur";
          blurwebBlurAreaEl.setAttribute("mask_id", "fig_mask_id_2000");
          blurwebBlurAreaEl.style.position = "absolute";
          blurwebBlurAreaEl.style.left = `${left}px`;
          blurwebBlurAreaEl.style.top = `${top}px`;
          blurwebBlurAreaEl.style.height = `${height}px`;
          blurwebBlurAreaEl.style.width = `${width}px`;
          //blurwebBlurAreaEl.innerHTML = blurAreaHTML;

          blurwebBlurAreaEl.appendChild(deleteBlurwebBlurAreaBtn);

          document.querySelector("body").appendChild(blurwebBlurAreaEl);

          deleteBlurwebBlurAreaBtn.addEventListener(
            "pointerdown",
            function (event) {
              //console.log("this should work");
              event.stopPropagation();
              if (deleteBlurwebBlurAreaBtn.parentNode) {
                chrome.storage.local.get("keepBlur", function (result) {
                  var url = document.URL;
                  var blurAreaId = deleteBlurwebBlurAreaBtn.parentNode.id;
                  removeBlurAreaInfoFromLocalStorage(url, blurAreaId);
                  deleteBlurwebBlurAreaBtn.parentNode.remove();
                });
              }
            }
          );
        }
      }
      document.body.style.visibility = "visible";
    } catch (error) {
      document.body.style.visibility = "visible";
    }
  });
}

const wrap = (wrapEleFn) => {
  const selection = document.getSelection();
  if (selection.rangeCount == 0) {
    return;
  }
  const range = selection.getRangeAt(0);
  let nodes = getTextNodesOfRange(range);

  let wrappEle = wrapEleFn();
  nodes.map(wrapNode(wrappEle, range));
  selection.removeAllRanges();
};

function removeBlurAreaInfoFromLocalStorage(url, id) {
  chrome.storage.local.get({ preBlurAreasInfo: [] }, function (result) {
    var preBlurAreasInfo = result.preBlurAreasInfo;

    for (var i = 0; i < preBlurAreasInfo.length; i++) {
      if (preBlurAreasInfo[i]["url"] === url) {
        if (preBlurAreasInfo[i]["id"] === id) {
          preBlurAreasInfo.splice(i, 1);
        }
      }
    }

    chrome.storage.local.set(
      { preBlurAreasInfo: preBlurAreasInfo },
      function () {}
    );
  });
}

(async function () {
  contentScript();

  let lastUrl = location.href;
  new MutationObserver(() => {
    const url = location.href;
    if (url !== lastUrl) {
      lastUrl = url;
      onUrlChange();
    }
  }).observe(document, { subtree: true, childList: true });

  function onUrlChange() {
    console.log("URL changed!", location.href);
    contentScript();
  }
})();
